@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
  * In addition, as a special exception, Dom Lachowicz
@@ -182,24 +182,32 @@ s_buildDictionaryDirs (std::vector<std::string> & dirs)
 	dirs.clear ();
 
 	char * config_dir = enchant_get_user_config_dir ();
-	dirs.push_back (g_build_filename (config_dir, "hunspell", nullptr));
+	gchar * tmp = g_build_filename (config_dir, "hunspell", nullptr);
+	dirs.push_back (tmp);
 	free (config_dir);
+	g_free(tmp);
 
 	for (const gchar* const * iter = g_get_system_data_dirs (); *iter; iter++)
 		{
-			dirs.push_back (g_build_filename (*iter, "hunspell", nullptr));
+			tmp = g_build_filename (*iter, "hunspell", nullptr);
+			dirs.push_back (tmp);
+			g_free(tmp);
 		}
 
 	/* Dynamically locate library and search for modules relative to it. */
 	char * enchant_prefix = enchant_get_prefix_dir();
 	if(enchant_prefix)
 		{
-			dirs.push_back (g_build_filename(enchant_prefix, "share", "enchant", "hunspell", nullptr));
-			g_free(enchant_prefix);
+			tmp = g_build_filename(enchant_prefix, "share", "enchant", "hunspell", nullptr);
+			dirs.push_back (tmp);
+			free(enchant_prefix);
+			g_free(tmp);
 		}
 
 #ifdef ENCHANT_HUNSPELL_DICT_DIR
-	dirs.push_back (enchant_relocate (ENCHANT_HUNSPELL_DICT_DIR));
+	config_dir = enchant_relocate (ENCHANT_HUNSPELL_DICT_DIR);
+	dirs.push_back (config_dir);
+	free(config_dir);
 #endif
 }
 
@@ -286,6 +294,7 @@ hunspell_request_dictionary (const char * tag)
 						g_dir_close (dir);
 						return dict;
 					}
+					g_free(dict);
 				}
 			}
 
@@ -306,6 +315,8 @@ HunspellChecker::requestDictionary(const char *szLang)
 	std::string aff(s_correspondingAffFile(dic));
 	if (s_fileExists(aff))
 	{
+		if (hunspell)
+			delete hunspell;
 		hunspell = new Hunspell(aff.c_str(), dic);
 	}
 	free(dic);
