@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
  * In addition, as a special exception, Dom Lachowicz
@@ -150,7 +150,7 @@ enchant_get_conf_dirs (void)
  error_exit:
 	free (pkgdatadir);
 	free (sysconfdir);
-	free (pkgconfdir);
+	g_free (pkgconfdir);
 	free (user_config_dir);
 	return NULL;
 }
