@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
  * In addition, as a special exception, Dom Lachowicz
@@ -207,7 +207,7 @@ EnchantPWL* enchant_pwl_init_with_file(const char * file)
 {
 	g_return_val_if_fail (file != NULL, NULL);
 
-	FILE* fd = g_fopen(file, "a+b");
+	FILE* fd = g_fopen(file, "a+");
 	if(fd == NULL)
 		return NULL;
 	fclose(fd);
@@ -232,7 +232,7 @@ static void enchant_pwl_refresh_from_file(EnchantPWL* pwl)
 	g_hash_table_destroy (pwl->words_in_trie);
 	pwl->words_in_trie = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
-	FILE *f = g_fopen(pwl->filename, "rb");
+	FILE *f = g_fopen(pwl->filename, "r");
 	if (!f) 
 		return;
 
@@ -248,10 +248,7 @@ static void enchant_pwl_refresh_from_file(EnchantPWL* pwl)
 			if(line_number == 1 && BOM == g_utf8_get_char(line))
 				line = g_utf8_next_char(line);
 
-			size_t l = strlen(line)-1;
-			if (line[l]=='\n') 
-				line[l] = '\0';
-			else if(!feof(f)) /* ignore lines longer than BUFSIZ. */ 
+			if(line[strlen(line)-1] != '\n' && !feof(f)) /* ignore lines longer than BUFSIZ. */ 
 				{
 					g_warning ("Line too long (ignored) in %s at line:%zu\n", pwl->filename, line_number);
 					while (NULL != (fgets (buffer, sizeof (buffer), f)))
@@ -261,7 +258,8 @@ static void enchant_pwl_refresh_from_file(EnchantPWL* pwl)
 						}
 					continue;
 				}
-						
+
+			g_strchomp(line);
 			if( line[0] && line[0] != '#')
 				{
 					if(g_utf8_validate(line, -1, NULL))
@@ -323,7 +321,7 @@ void enchant_pwl_add(EnchantPWL *pwl,
 
 	if (pwl->filename != NULL)
 	{
-		FILE *f = g_fopen(pwl->filename, "a+b");
+		FILE *f = g_fopen(pwl->filename, "a+");
 		if (f)
 			{
 				/* Since this function does not signal I/O
@@ -966,7 +964,7 @@ static EnchantTrieMatcher* enchant_trie_matcher_init(const char* const word,
 	EnchantTrieMatcher* matcher = g_new(EnchantTrieMatcher,1);
 	matcher->num_errors = 0;
 	matcher->max_errors = maxerrs;
-	matcher->word = g_new0(char,len+maxerrs+1); // Ensure matcher does not overrun buffer
+	matcher->word = g_new0(char,len+maxerrs+2); // Ensure matcher does not overrun buffer: +2 for transpose check
 	strcpy(matcher->word, pattern);
 	g_free(pattern);
 	matcher->word_pos = 0;
